package net.locusworks.common.utils;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import static net.locusworks.common.utils.Checks.checkArguments;
import static net.locusworks.common.utils.Checks.checkNotNull;

public class Splitter {
  
  private String splitSeq;
  private boolean omitEmptyStrings = false;
  private int partition;
  private int limit;
  
  private static Splitter splitter;
  
  private Splitter(String seq) {
    this.splitSeq = seq;
  }
  
  private Splitter(int partition) {
    this.partition = partition;
  }
  
  public Splitter omitEmptyStrings() {
    this.omitEmptyStrings = true;
    return this;
  }
  
  public Splitter withLimit(int limit) {
    this.limit = limit;
    return this;
  }
  
  public MapSplitter withKeyValueSeparator(String separator) {
    checkArguments(!Utils.isEmptyString(separator), "Key value separator cannot be empty or null");
    return new MapSplitter(this, separator);
  }
  
  public String[] splitToArray(String sentence) {
    List<String> list = split(sentence);
    return list.toArray(new String[list.size()]);
  }
  
  public List<String> split(String sentence) {
    checkArguments(!Utils.isEmptyString(sentence), "provided value is null or empty");
    List<String> list = new ArrayList<>();
    
    if (!Utils.isEmptyString(splitSeq))
      populateForTrimmer(sentence, list);
    else 
      populateForFixedWidth(sentence, list);
    
    return limit > 0 ? list.subList(0, limit) : list;
  }

  private void populateForFixedWidth(String sentence, List<String> list) {
    int strLength = sentence.length();
    for (int i = 0; i < strLength; i += partition) {
      list.add(sentence.substring(i, Math.min(strLength, i + partition)));
    }
  }

  private void populateForTrimmer(String sentence, List<String> list) {
    for (String s : sentence.split(splitSeq)) {
      if (s == null || (omitEmptyStrings && s.trim().isEmpty())) continue;
      list.add(s.trim());
    }
  }
  
  public static Splitter fixedLengthSplit(int partition) {
    checkArguments(partition > 0, "Partition has to be greater than 0");
    splitter = new Splitter(partition);
    return splitter;
  }
  
  public static Splitter on(String split) {
    checkNotNull(split, "Split value provided was null");
    splitter = new Splitter(split);
    return splitter;
  }
  
  public static Splitter onNewLine() {
    return on("\\r?\\n");
  }
  
  public static Splitter onSpace() {
    return on(" ");
  }
  
  public static class MapSplitter {
    
    private Splitter splitter;
    private String separator;
    private boolean skipInvalid = false;
    
    private MapSplitter(Splitter splitter, String separator) {
      checkNotNull(splitter, "Splitter cannot be null");
      checkArguments(!Utils.isEmptyString(separator), "Key value separator cannot be empty or null");
      this.splitter = splitter;
      this.separator = separator;
    }
    
    public MapSplitter skipInvalidKeyValues() {
      this.skipInvalid = true;
      return this;
    }
    
    public Map<String, String> split(String sentence) {
      checkArguments(!Utils.isEmptyString(sentence), "provided value is null or empty");
      Map<String, String> map = new LinkedHashMap<>();
      
      for (String s : splitter.split(sentence)) {
        String[] keyValue = s.split(separator);
        try {
          checkArguments(keyValue.length == 2, "invalid length found for key value mapping");
        } catch (IllegalArgumentException ex) {
          if (!skipInvalid) throw ex;
          continue;
        }
        map.put(keyValue[0], keyValue[1]);
      }
      
      return map;
    }
  }
}
